"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WinixService = void 0;
const winix_1 = require("../winix");
const encryption_1 = require("../encryption");
class WinixService {
    constructor(storagePath, encryptionKey) {
        this.encryptionKey = encryptionKey;
        this.winix = new winix_1.WinixHandler(storagePath, encryptionKey);
        this.hasValidAuth = false;
    }
    async init(auth) {
        this.hasValidAuth = false;
        if (!auth) {
            return { needsLogin: true };
        }
        const refreshToken = await this.winix.getRefreshToken();
        if (!refreshToken) {
            return { needsLogin: true };
        }
        try {
            await this.winix.refresh(auth);
            this.hasValidAuth = true;
        }
        catch (e) {
            this.hasValidAuth = false;
            return { needsLogin: true };
        }
        return { needsLogin: false };
    }
    async login({ email, password }) {
        const result = await this.winix.login(email, password);
        this.hasValidAuth = true;
        const encryptedPassword = await (0, encryption_1.encrypt)(password, this.encryptionKey);
        return { username: email, password: encryptedPassword, userId: result.userId };
    }
    async discoverDevices() {
        if (!this.hasValidAuth) {
            throw new winix_1.UnauthenticatedError();
        }
        let winixDevices;
        try {
            winixDevices = await this.winix.getDevices();
        }
        catch (e) {
            if (e instanceof winix_1.UnauthenticatedError) {
                throw new winix_1.UnauthenticatedError();
            }
            throw e;
        }
        const devices = winixDevices.map(({ deviceAlias, deviceId, modelName }) => {
            return { deviceId, deviceAlias, modelName };
        });
        return { devices };
    }
}
exports.WinixService = WinixService;
